#!/usr/bin/env python
#
# Data API:
# http://api.stackexchange.com/docs/questions#page=1&order=desc&min=2&sort=votes&tagged=application-development&filter=default&site=askubuntu&run=true
#

import os
import json
import time

from urllib2 import urlopen, Request
from zlib import decompress, MAX_WBITS


# So sue me. They're global
MIN_VOTES = 2
API_ROOT = 'http://api.stackexchange.com/2.1'
TAGS = ['application-development', 'scopes']
EX_TAGS = set(['gtk', 'gtk3', 'pygtk', 'python', 'c#', 'java', 'glade', 'quickly', 'qml'])
SITE = 'askubuntu'
PAGE = 1

HTML_OUT = os.path.join(os.path.dirname(__file__), 'scopes.html')
JSON_OUT = os.path.join(os.path.dirname(__file__), 'scopes.json')
current_run = {
    'keep': list(),
    'data': dict(),
    'skip': list(),
    'sections': dict(),
}

SECTIONS = {
's': {'title': 'General Scope Development', 'subsections': {}, },
'c': {'title': 'Category Placement', 'subsections': {}, },
'p': {'title': 'Previews & Actions', 'subsections': {}, },
}

def load_previous():
    global current_run
    if os.path.exists(JSON_OUT):
        json_fh = open(JSON_OUT)
        current_run = json.loads(json_fh.read())
        json_fh.close()
    else:
        print "No file at %s" % JSON_OUT

def save_run():
    global current_run
    json_fh = open(JSON_OUT, 'w')
    json_fh.write(json.dumps(current_run))
    json_fh.close()
    
def build_query():
    q = [
        'pagesize=100',
        'order=desc',
        'min=%s' % MIN_VOTES,
        'sort=votes',
        'tagged=%s' % ';'.join(TAGS), 
        'filter=!9hnGsreuM',
         'page=%s' % PAGE,
        ]
    return '&'.join(q)

def site(url_provider):
    def wrapper(*args, **kargs):
        url = url_provider(*args, **kargs)
        if '?' in url:
            return url + '&site=' + SITE
        else:
            return url + '?site=' + SITE
    return wrapper
    
@site
def build_questions_url():
    return '%s/questions?%s' % (API_ROOT, build_query())
@site
def build_details_url(question_id):
    return '%s/questions/%s' % (API_ROOT, question_id)

def fetch_contents():
    global PAGE
    contents = []
    has_more = True

    while has_more:
        req = Request(build_questions_url())
        resp = urlopen(req).read()
        results = json.loads(decompress(resp, 16 + MAX_WBITS).decode('UTF-8'))
        contents = contents + results['items']
        if results['quota_remaining'] < 5:
            raise Exception('Quota too low to continue. Please wait')

        if 'backoff' in results:
            time.sleep(data['backoff'] + 2)

        has_more = results['has_more']
        PAGE += 1

    return contents

def show_question(question):
    global current_run
    
    answer = None
    print '(%(score)s) %(title)s' % question
    while not answer:
        answer = raw_input('[Y/n/m/t/r]? ')
        if answer and answer[0].lower() == 'y':
            set_section(question)
            current_run['data'][str(question['question_id'])] = {'id': question['question_id'], 'title': question['title'], 'link': question['link']}
            current_run['keep'].append(question['question_id'])
        elif answer and answer[0].lower() == 'n':
            current_run['skip'].append(question['question_id'])
            break
        elif answer and answer[0].lower() == 'm':
            print question['body']
            answer = None
        elif answer and answer[0].lower() == 't':
            print ', '.join(question['tags'])
            answer = None
        elif answer and answer[0].lower() == 'r':
            print '\n(%(score)s) %(title)s' % question
            answer = None
        elif answer and answer[0].lower() == 'i':
            break
        elif answer and answer[0].lower() == 'o':
            os.system('xdg-open %s' % question['link'])
            answer = None
        elif answer and answer[0].lower() == 'q':
            quit_run()
        elif answer and answer[0].lower() == '?':
            print '''
Y = Yes, use this question
n = No, skip this question
m = More info, show the question body
t = Show tags
r = Repeat the question
i = Ignore the question for now
o = Open question in a browser
q = Quit without saving
            ''' % question
            answer = None
        elif answer:
            print '\n Unknown option: \'%s\', enter \'?\' for options' % answer
            answer = None
    
def set_section(question):
    global current_run
    
    answer = None
    print 'What section?'
    while not answer:
        answer = raw_input('[%s]? ' % '/'.join(SECTIONS.keys()))
        if answer and answer[0].lower() == '?':
            for section_char, section in SECTIONS.items():
                print '%s =  %s' % (section_char, section['title'])
            answer = None
        elif answer and answer[0].lower() == 'q':
            quit_run()
        elif answer and answer[0].lower() in SECTIONS:
            if answer[0].lower() not in current_run['sections']:
                current_run['sections'][answer[0].lower()] = list()
            current_run['sections'][answer[0].lower()].append(question['question_id'])
        elif answer:
            print '\n Unknown option: \'%s\', enter \'?\' for options' % answer
            answer = None
    
def quit_run():
    save_run()
    html_fh = open(HTML_OUT, 'w')
    for section_char, section in SECTIONS.items():
        if section_char in current_run['sections']:
            html_fh.write('<h2>%s</h2>\n<ul>\n' % section['title'])
            for question_id in current_run['sections'][section_char]:
                question = current_run['data'][str(question_id)]
                html_fh.write('    <li><a href="%(link)s" target="_blank">%(title)s</a></li>\n' % question)
            html_fh.write('</ul>\n')
            html_fh.write('<div class="box-orange clearfix"><h3 style="margin: 0 0 8px 0; padding-top: 0; border-top: none;">Still have questions?</h3><a class="link-action-small" href="http://www.askubuntu.com/questions/ask?tags=application-development" target="_blank"><span class="cta-left"></span><span class="cta-center">Ask your Question</span><span class="cta-right"></span></a><span>If you could not find an answer to your question here, you can ask the thousands of Ubuntu users and developers on Ask Ubuntu.</span></div>\n\n')
    html_fh.close()
    exit(0)
    
if __name__ == '__main__':
    data = fetch_contents()

    load_previous()
    alltags = dict()
    for question in data:
        if question['is_answered'] and not 'closed_date' in question:
            if len(EX_TAGS & set(question['tags'])) > 0:
                continue
            if question['question_id'] in current_run['keep']:
                # Update the question data in case it was changed on AU
                current_run['data'][str(question['question_id'])] = {'id': question['question_id'], 'title': question['title'], 'link': question['link']}
                continue
            if question['question_id'] in current_run['skip']:
                continue
                
            show_question(question)
    quit_run()
